{--------------------------------------------------------}
{                                                        }
{    Programa leitor de notcias e RSS                   }
{                                                        }
{    Folheamento de RSS                                  }
{                                                        }
{    Autor: Jos Antonio Borges e Fabiano Ferreira       }
{                                                        }
{    Em maio/2013                                        }
{                                                        }
{--------------------------------------------------------}

unit nenavega;

interface

uses
    windows,
    sysutils,
    classes,
    dvcrt,
    dvwin,
    dvform,
    dvexec,
    nerede,
    nevars,
    nerss,
    neutil,
    neleit,
    nemsg,
    neatom,
    shellapi;

procedure navegaNosSitesRss (nomeCategoria: string);

implementation

{--------------------------------------------------------}
{                     chama o navegador                  }
{--------------------------------------------------------}

procedure chamaNavegador (site: string);
var nomeProg: string;
begin

    nomeProg := sintAmbiente ('VOXNEWS', 'NAVEGADOR');
    if nomeProg = '' then
        nomeProg := 'c:\winvox\webvox.exe';

    if executaProg (nomeProg, '', site) >= 32 then
        begin
            esperaProgVoltar;
            while sintFalando do waitMessage;
        end
    else
        mensagem ('NEERRCHM', 1);  {'Erro ao chamar o leitor'}
end;

{--------------------------------------------------------}
{                   compila o RSS                        }
{--------------------------------------------------------}

function compilaRss (nomeSite: string; var canal: TRss): boolean;
var status: integer;
    rssLido: string;
    c, c2: char;
    ok, ehAtom: boolean;

begin
    result := false;
    rsslido := HttpDownload (nomeSite, status);

    if status <> 200 then
        begin
            mensagem ('NENAOCRG', 1);   {'No consegui carregar o RSS.'}
            mensagem ('NESTATUS', 1);   {'Status: '}
            sintWriteln (intToStr(status));

            if status = -1 then
                mensagem ('NEINTCAI', 1)    {'Internet parece ter cado.'}
            else
            if status >= 400  then
                mensagem ('NESITEFO', 1);   {'Site parece estar fora do ar.'}

            exit;
        end;

    inicXmlParserFromString (rsslido);
    if not ehArquivoRss (ehAtom) then
        begin
            mensagem ('NERSSINV', 1);  {'Desculpe, mas isso no  um arquivo Rss vlido.'}
            exit;
        end;

    if ehAtom then
        ok := carregaAtomRss (canal)
    else
        ok := carregaRss (canal);

    if not ok then
        begin
            limpaBufTec;
            writeln;
            mensagem ('NEXMLERR', 0);  {'Este RSS no  totalmente compatvel com este programa.  Aceito? '}
            sintLeTecla (c, c2);
            writeln;
            if upcase (c) = 'S' then
                result := true;
        end
    else
        result := true;
end;

{--------------------------------------------------------}
{                         ajuda                          }
{--------------------------------------------------------}

procedure ajuda;
begin
    mensagem ('NEOPCAO', 1);   {'As opes so:'}
    mensagem ('NEOPN01', 1);   {'ENTER - abre a pgina'}
    mensagem ('NEOPN02', 1);   {'Control ENTER: executa a pgina com o navegador do Windows'}
//    mensagem ('NEOPN03', 1);   {'L  - leitura rpida desta pgina'}
    mensagem ('NEOPN04', 1);   {'I  - mostra informaes sobre esta pgina'}
    mensagem ('NEOPN05', 1);   {'D  - mostra detalhes deste canal'}
    mensagem ('NEOPN06', 1);   {'C  - pe o endereo desta pgina na rea de transferncia'}
    mensagem ('NEOPN07', 1);   {'T  - pe o ttulo desta pgina na rea de transferncia'}
    mensagem ('NEOPN20', 1);   {'F9 - seleciona as opes com as setas'}
    mensagem ('NEAJCN99',1);   {'ESC - terminar'}
    writeln;
    mensagem ('NEAPTTEC', 1);  {'Aperte uma tecla para continuar...'}

    readkey;
    limpaBufTec;
end;

{--------------------------------------------------------}
{             seleo interativa de opes               }
{--------------------------------------------------------}

function selSetasOpcao: char;

    procedure MenuAdiciona (msg: string);
    begin
         popupMenuAdiciona (msg, pegaTextoMensagem (msg));
    end;

var n: integer;
const
    tabLetrasOpcao: string = ENTER + CTLENTER + 'IDCT' + ESC;

begin
    garanteEspacoTela (8);
    popupMenuCria (wherex, wherey, 60, length(tabLetrasOpcao), MAGENTA);
    MenuAdiciona ('NEOPN01');   {'ENTER - abre a pgina'}
    MenuAdiciona ('NEOPN02');   {'Control ENTER: executa a pgina com o navegador do Windows'}
//    MenuAdiciona ('NEOPN03');   {'L  - leitura rpida desta pgina'}
    MenuAdiciona ('NEOPN04');   {'I  - mostra informaes sobre esta pgina'}
    MenuAdiciona ('NEOPN05');   {'D  - mostra detalhes deste canal'}
    MenuAdiciona ('NEOPN06');   {'C  - pe o endereo desta pgina na rea de transferncia'}
    MenuAdiciona ('NEOPN07');   {'T  - pe o ttulo desta pgina na rea de transferncia'}
    MenuAdiciona ('NEAJCN99');  {'ESC - terminar'}
    n := popupMenuSeleciona;
    if n > 0 then
        begin
            selSetasOpcao := tabLetrasOpcao[n];
            writeln (tabLetrasOpcao[n]);
        end
    else
        selSetasOpcao := ESC;
end;

{--------------------------------------------------------}
{             mostra detalhes deste RSS                  }
{--------------------------------------------------------}

procedure mostraDetalhesRss (canal: TRss);
begin
    clrScr;
    textBackGround (RED);
    writeln (canal.title);
    textBackGround (BLACK);
    writeln;
    sintWriteln (canal.description);

    limpaBufTec;
    mensagem ('NEUSESET', 0);   {'Use as setas, ao final tecle ESC'}
    formCria;
    with canal do
        begin
            formCampo('', 'Ttulo', title, 255);
            formCampo('', 'Endereo', link, 255);
            if language <> '' then
                formCampo('', 'Lngua', language, 255);
            if copyright <> '' then
                formCampo('', 'Copyright', copyright, 255);
            if managingEditor <> '' then
                formCampo('', 'Editor gerente', managingEditor, 255);
            if webMaster <> '' then
                formCampo('', 'Webmaster', webMaster, 255);
            if pubDate <> '' then
                formCampo('', 'Data de publicao', pubDate, 255);
            if lastBuildDate <> '' then
                formCampo('', 'Data de montagem',  lastBuildDate, 255);
            if category <> '' then
                formCampo('', 'Categoria', category, 255);
            if generator <> '' then
                formCampo('', 'Gerador', generator, 255);
            if docs <> '' then
                formCampo('', 'Documentos', docs, 255);
            if cloud <> '' then
                formCampo('', 'Nuvem', cloud, 255);
            if ttl <> 0 then
                formCampoInt ('', 'Validade', ttl);
            if image.title <> '' then
                formCampo('', 'Imagem', image.title, 255);
            if rating <> '' then
                formCampo('', 'Idade recomendada', rating, 255);
            if textInput <> '' then
                formCampo('', 'Texto', textInput, 255);
            if skipHours <> '' then
                formCampo('', 'Horas passadas', skipHours, 255);
            if skipDays <> '' then
                formCampo('', 'Dias passados', skipDays, 255);
            formEdita(false);
        end;

    limpaBufTec;
end;

{--------------------------------------------------------}
{             mostra detalhes de um item                 }
{--------------------------------------------------------}

procedure mostraDetalhesSite (item: TItem);
begin
    clrScr;
    textBackGround (RED);
    writeln (item.title);
    textBackGround (BLACK);
    writeln;
    sintWriteln (removeTagsHTML(item.description));

    limpaBufTec;
    mensagem ('NEUSESET', 0);   {'Use as setas, ao final tecle ESC'}
    formCria;
    with item do
        begin
            formCampo('', 'Ttulo', title, 255);
            formCampo('', 'Endereo', link, 255);
            formCampo('', 'Autor', author, 255);
            formCampo('', 'Categoria', category, 255);
            formCampo('', 'Comentrio', comments, 255);
            formCampo('', 'Pertinncia', enclosure, 255);
            formCampo('', 'Identificao', guid, 255);
            formCampo('', 'Data de publicao', pubDate, 255);
            formCampo('', 'Data de modificao', modifiedDate, 255);
            formCampo('', 'Fonte', source, 255);
            formCampo('', 'Seo', section, 255);
            formEdita(false);
        end;

    limpaBufTec;
end;

{--------------------------------------------------------}
{             abre o site com Webvox                     }
{--------------------------------------------------------}

procedure abreSite (site: string);
var
    nomeProg: string;
begin
    writeln (site);
    writeln;
    mensagem ('NECHANAV', 1);  {'Chamando navegador'}

    nomeProg := sintAmbiente ('VOXNEWS', 'NAVEGADOR');
    if nomeProg = '' then
        nomeProg := 'c:\winvox\webvox.exe';

    if executaProg (nomeProg, '', site) >= 32 then
        begin
            esperaProgVoltar;
            while sintFalando do waitMessage;
        end
    else
        mensagem ('NEERRNAV', 1);  {'Erro ao chamar o navegador'}
end;

{--------------------------------------------------------}
{            Descobre qual  o navegador padro
{--------------------------------------------------------}

function GetDefaultBrowser: string;
var
    tmp : PChar;
    res : PChar;
begin
    tmp := StrAlloc(255);
    res := StrAlloc(255);
    try
        GetTempPath(255,tmp);
        FileCreate(tmp+'htmpl.htm');
        FindExecutable('htmpl.htm',tmp,Res);
        Result := ExtractFilePath(res) + ExtractFileName(res);
        SysUtils.DeleteFile(tmp+'htmpl.htm');
    finally
        StrDispose(tmp);
        StrDispose(res);
    end;
end;

{--------------------------------------------------------}
{       mostra o youtube no navegador web padro
{--------------------------------------------------------}

procedure mostraWeb (pagina: string);
begin
    mensagem ('NEABNAV', 1);  {'Abrindo navegador. Acione ALT F4 quando terminar.'}
    while sintFalando do waitMessage;
    delay (100);

    executaProg(GetDefaultBrowser, '', pagina);
    delay (5000);
    while getForegroundWindow <> crtWindow do delay (500);
end;

{--------------------------------------------------------}
{  pe informaes sobre link na rea de transferncia   }
{--------------------------------------------------------}

procedure poeAreaTransf (s: string);
begin
    putClipboard (pchar(s));
end;

{--------------------------------------------------------}
{        interao para escolha de sites a exibir        }
{--------------------------------------------------------}

procedure exibeSitesDoRSS (canal: TRSS; nomeCategoria, nomeCanal: string);
var
    site, titulo: string;
    i, itemInic: integer;
    c, c2: char;
    primeiraVez: boolean;

label execOpcao, termina;
begin
    itemInic := 1;
    primeiraVez := true;
    repeat
        clrscr;
        TextBackground (BLUE);
        writeln ('VoxNews - ' + nomeCategoria + ' - ' + nomeCanal);
        writeln;
        TextBackground (RED);
        if primeiraVez then
            mensagem ('NEFOLSIT', 1)  {'Folheie os sites com as setas, F1 ajuda'}
        else
            writeln (pegaTextoMensagem ('NEFOLSIT'));
        primeiraVez := false;
        TextBackground (BLACK);

        folheiaCria (wherex, wherey, 80, 25);
        for i := 0 to length(canal.items)-1 do
            folheiaAdiciona(canal.items[i].title);
        folheiaExecuta(itemInic, itemInic, c, c2, true);
        folheiaLimpa;
        folheiaDestroi;

termina:
        if c = ESC then
            begin
                mensagem ('NEFOLTRM', 1);   {'Folheamento terminado'}
                break;
            end;

execOpcao:
        site := canal.items[itemInic-1].link;
        titulo := canal.items[itemInic-1].title;

        case upcase(c) of
          ENTER: abreSite (site);
       CTLENTER: mostraWeb (site);
     //     'L': leituraRapidaHTML (site);
            'I': mostraDetalhesSite (canal.items[itemInic-1]);
            'D': mostraDetalhesRss (canal);
            'T': poeAreaTransf (titulo);
            'C': poeAreaTransf (site);
            ESC: goto termina;
            #0:  case c2 of
                     F1: ajuda;
                     F9: begin
                             c := selSetasOpcao;
                             goto execOpcao;
                         end;

                 end;
        else
            mensagem ('NEOPINV', 1);   {'Opo invlida'}
        end;

        clreol;
        mensagem ('NECNTFOL', 0); {'Continue folheando'}
            
    until false;
    folheiaDestroi;
end;

{--------------------------------------------------------}
{          escolhe um dos canais para navegar            }
{--------------------------------------------------------}

function escolheCanal (nomeCategoria: string;
                       var nomeCanal, siteCanal: string): boolean;
var
    itens: array [0..8000] of char;
    contItem: array [1..200] of string;
    p: pchar;
    nitens: integer;
    umItem: array [0..255] of char;
    n: integer;
begin
    clrscr;
    TextBackground (BLUE);
    writeln ('VoxNews - ' + nomeCategoria);
    writeln;
    mensagem ('NEESCCAN', 1);  {'Escolha o canal desejado e aperte enter'}
    TextBackground(BLACK);

    getprivateProfileString (pchar(nomeCategoria), NIL, '', itens, 8000, pchar(arqIndice));
    p := itens;
    nitens := 0;

    popupMenuCria(1, wherey, 79, 26-wherey, RED);
    while p^ <> #$0 do
        begin
            nitens := nitens + 1;
            getprivateProfileString (pchar(nomeCategoria), p, '', umItem, 255, pchar(arqIndice));
            contItem[nitens] := strPas (umItem);
            popupMenuAdiciona ('', strPas (p));
            p := p + strlen(p) + 1;
        end;
    n := popupMenuSeleciona;
    if (n <= 0) or (n > nitens) then
        begin
            nomeCanal := '';
            siteCanal := '';
            result := false;
        end
    else
        begin
            limpaBaixo (2);
            gotoxy (1, 2);
            textColor (YELLOW);
            writeln (contItem[n]);
            textColor (WHITE);

            nomeCanal := opcoesItemSelecionado;
            siteCanal := contItem[n];
            result := true;
        end;
end;

{--------------------------------------------------------}
{            interao para escolher as pginas          }
{--------------------------------------------------------}

procedure navegaNosSitesRss (nomeCategoria: string);
var nomeCanal, paginaRss: string;
    canal: TRss;
begin
    while escolheCanal (nomeCategoria, nomeCanal, paginaRss) do
        begin
            if compilaRSS (paginaRss, canal) then
                 begin
                     exibeSitesDoRSS (canal, nomeCategoria, nomeCanal);
                     liberaRss(canal);
                 end;
        end;

    mensagem ('NECATTRM', 1);  {'Categoria terminada'}
end;

end.


